#    Deep Space - A space-shooting game.
#    Copyright (C) 2014  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import pygame
pygame.init()

import copy

from code.classes.AnimatedSprite import AnimatedSprite
from code.classes.Projectile import Projectile
from code.classes.Sentence import Sentence


class PlayerSpaceship( AnimatedSprite ):

	def __init__( self, center_xy_position, movement_speed, controls,
		transparency_colorkey, images, interval, interval_stepping,
		current_projectile_level, projectile_levels, health, score,
		projectile_data, score_topright_coords, health_topleft_coords,
		projectile_topleft_coords,
		dead, movement_boundaries, interval_repeat=True, alternate=False ):
		
		super( PlayerSpaceship, self ).__init__( images, transparency_colorkey,
		 interval, interval_stepping, interval_repeat=True, alternate=False)

		self.rect = self.image.get_rect( center = center_xy_position )

		# Controls.
		self.controls = controls

		# Movement speed.
		self.movement_speed = movement_speed

		# Data for projectiles.
		self.projectile_data = projectile_data

		# Projectiles that have been fired.
		self.projectiles = pygame.sprite.Group()

		self.current_projectile_level = current_projectile_level
		self.projectile_levels = projectile_levels

		self.transparency_colorkey = transparency_colorkey


		self.score = score
		self.score_top_coord = score_topright_coords[0]
		self.score_right_coord = score_topright_coords[1]
		
		self.health = health
		self.health_top_coord = health_topleft_coords[0]
		self.health_left_coord = health_topleft_coords[1]

		self.projectile_top_coord = projectile_topleft_coords[0]
		self.projectile_left_coord = projectile_topleft_coords[1]

		self.score_display = Sentence( "SCORE: " + str( self.score ),
			self.score_top_coord, self.score_right_coord )

		self.health_display = Sentence( "HEALTH: " + str( self.health ) + " %",
			self.health_top_coord, self.health_left_coord )

		self.projectile_level_display = Sentence( "PROJECTILE LEVEL: " + str(
			self.current_projectile_level ), self.projectile_top_coord,
			self.projectile_left_coord )


		self.dead = dead
		self.movement_boundaries = movement_boundaries


	def update( self, eventlist ):
	
		super( PlayerSpaceship, self ).update()

		if not self.dead:
			self.handle_controls( eventlist )


	def handle_controls( self, eventlist ):

		for e in eventlist:
			if e.type == pygame.KEYDOWN:
				if e.key == self.controls["move_left"][0]:
					self.controls["move_left"][1] = True
				elif e.key == self.controls["move_right"][0]:
					self.controls["move_right"][1] = True
				elif e.key == self.controls["move_up"][0]:
					self.controls["move_up"][1] = True
				elif e.key == self.controls["move_down"][0]:
					self.controls["move_down"][1] = True
				elif e.key == self.controls["fire"][0]:
					self.controls["fire"][1] = True
					
			if e.type == pygame.KEYUP:
				if e.key == self.controls["move_left"][0]:
					self.controls["move_left"][1] = False
				elif e.key == self.controls["move_right"][0]:
					self.controls["move_right"][1] = False
				elif e.key == self.controls["move_up"][0]:
					self.controls["move_up"][1] = False
				elif e.key == self.controls["move_down"][0]:
					self.controls["move_down"][1] = False


		if self.controls["move_left"][1]:
			self.move_left()
		if self.controls["move_right"][1]:
			self.move_right()
		if self.controls["move_up"][1]:
			self.move_up()
		if self.controls["move_down"][1]:
			self.move_down()
		if self.controls["fire"][1]:
			self.fire()
			self.controls["fire"][1] = False # A continous fire is not possible.


	def move_left( self ):
		if self.rect.left > self.movement_boundaries.left:
			self.rect.move_ip( -self.movement_speed, 0 )


	def move_right( self ):
		if self.rect.right < self.movement_boundaries.right:
			self.rect.move_ip( self.movement_speed, 0 )


	def move_up( self ):
		if self.rect.top > self.movement_boundaries.top:
			self.rect.move_ip( 0, -self.movement_speed )


	def move_down( self ):
		if self.rect.bottom < self.movement_boundaries.bottom:
			self.rect.move_ip( 0, self.movement_speed )


	def fire( self ):
		p_data = copy.deepcopy( self.projectile_data )
		p_data["center_xy_position"] = self.rect.center

		projectile_level = self.projectile_levels[self.current_projectile_level]
		
		for projectile_angle in projectile_level:
			p_data["angle"] = projectile_angle
			self.projectiles.add( Projectile( **p_data ) )


	def score_bonus( self, amount ):
		self.score += amount
		self.update_score()


	def increase_health( self, amount ):
		self.health += amount
		if self.health > 100:
			self.health = 100


	def weapon_upgrade( self, amount ):
		self.current_projectile_level += 1
		if self.current_projectile_level > len( self.projectile_levels ):
			self.current_projectile_level = len( self.projectile_levels )

			self.update_projectile()


	def update_score( self ):
		self.score_display = Sentence( "SCORE: " + str( self.score ),
			self.score_top_coord, self.score_right_coord )


	def update_health( self ):
		self.health_display = Sentence( "HEALTH: " + str( self.health ) + " %",
			self.health_top_coord, self.health_left_coord )


	def update_projectile( self ):
		self.projectile_level_display = Sentence( "PROJECTILE LEVEL: " + str(
			self.current_projectile_level ), self.projectile_top_coord,
			self.projectile_left_coord )		
